package com.gx.obe.component.form;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.function.Consumer;
import java.util.function.Function;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Text;
import org.eclipse.wb.swt.SWTResourceManager;

public class FForm<T> extends Composite {
	
	private final IForm<T> iForm;
	
	private final List<Consumer<T>> consumerList = new ArrayList<>();

	private int numColumns;
	
	public FForm(Composite parent, int numColumns) {
		super(parent, SWT.NONE);
		this.numColumns = numColumns;
		
		setBackground(SWTResourceManager.getColor(215, 215, 215));
		setBackgroundMode(SWT.INHERIT_FORCE);
		GridLayout gridLayout = new GridLayout(numColumns * 2, false);
		gridLayout.verticalSpacing = 1;
		gridLayout.horizontalSpacing = 1;
		gridLayout.marginHeight = 1;
		gridLayout.marginWidth = 1;
		setLayout(gridLayout);
		
		iForm = new IForm<T>() {
			
			@Override
			public void addNull() {
				CLabel lblNewLabel = new CLabel(FForm.this, SWT.RIGHT);
				lblNewLabel.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
				GridData gd_lblNewLabel = new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1);
				gd_lblNewLabel.heightHint = 30;
				gd_lblNewLabel.widthHint = 100;
				lblNewLabel.setLayoutData(gd_lblNewLabel);
				
				CLabel lblNewLabel_1 = new CLabel(FForm.this, SWT.NONE);
				lblNewLabel_1.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
				GridData gd_lblNewLabel_1 = new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1);
				gd_lblNewLabel_1.heightHint = 30;
				gd_lblNewLabel_1.widthHint = 200;
				lblNewLabel_1.setLayoutData(gd_lblNewLabel_1);
			}
			
			public void add(String title, Function<T, String> getValueFun) {
				CLabel lblNewLabel = new CLabel(FForm.this, SWT.RIGHT);
				lblNewLabel.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
				GridData gd_lblNewLabel = new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1);
				gd_lblNewLabel.heightHint = 30;
				gd_lblNewLabel.widthHint = 100;
				lblNewLabel.setLayoutData(gd_lblNewLabel);
				lblNewLabel.setText(title);
				
				CLabel lblNewLabel_1 = new CLabel(FForm.this, SWT.NONE);
				lblNewLabel_1.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
				GridData gd_lblNewLabel_1 = new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1);
				gd_lblNewLabel_1.heightHint = 30;
				gd_lblNewLabel_1.widthHint = 200;
				lblNewLabel_1.setLayoutData(gd_lblNewLabel_1);
				consumerList.add(t -> lblNewLabel_1.setText(Optional.ofNullable(t).map(getValueFun).orElse("")));
			}
			
			public void addLong(String title, Function<T, String> getValueFun) {
				CLabel lblNewLabel = new CLabel(FForm.this, SWT.RIGHT);
				lblNewLabel.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
				GridData gd_lblNewLabel = new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1);
				gd_lblNewLabel.heightHint = 30;
				lblNewLabel.setLayoutData(gd_lblNewLabel);
				lblNewLabel.setText(title);
				
				CLabel lblNewLabel_1 = new CLabel(FForm.this, SWT.NONE);
				lblNewLabel_1.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
				GridData gd_lblNewLabel_1 = new GridData(SWT.FILL, SWT.CENTER, true, false, numColumns * 2 - 1, 1);
				gd_lblNewLabel_1.heightHint = 30;
				lblNewLabel_1.setLayoutData(gd_lblNewLabel_1);
				consumerList.add(t -> lblNewLabel_1.setText(Optional.ofNullable(t).map(getValueFun).orElse("")));
			}
			
			public void addBig(String title, Function<T, String> getValueFun, int height) {
				CLabel lblNewLabel = new CLabel(FForm.this, SWT.RIGHT);
				lblNewLabel.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
				GridData gd_lblNewLabel = new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1);
				gd_lblNewLabel.heightHint = height;
				lblNewLabel.setLayoutData(gd_lblNewLabel);
				lblNewLabel.setText(title);
				
				Composite composite = new Composite(FForm.this, SWT.NONE);
				composite.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
				GridData gd_composite = new GridData(SWT.FILL, SWT.FILL, true, false, numColumns * 2 - 1, 1);
				gd_composite.heightHint = height;
				composite.setLayoutData(gd_composite);
				GridLayout gl_composite = new GridLayout(1, false);
				gl_composite.marginWidth = 1;
				composite.setLayout(gl_composite);
				
				Text lblNewLabel_1 = new Text(composite, SWT.WRAP | SWT.V_SCROLL);
				lblNewLabel_1.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
				lblNewLabel_1.setEditable(false);
				consumerList.add(t -> lblNewLabel_1.setText(Optional.ofNullable(t).map(getValueFun).orElse("")));
			}

		};
		
	}
	
	public void init(Consumer<IForm<T>> consumer) {
		consumer.accept(iForm);	
		Composite composite_1 = new Composite(this, SWT.NONE);
		GridData gd_lblNewLabel = new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1);
		gd_lblNewLabel.heightHint = 0;
		composite_1.setLayoutData(gd_lblNewLabel);
		composite_1.setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		composite_1.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, numColumns * 2, 1));
	}
	
	public void refresh(T t) {
		consumerList.forEach(c -> c.accept(t));
	}
	
	@Override
	protected void checkSubclass() {}
}
