package com.gx.obe.component.grid;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.function.Function;

import org.eclipse.nebula.widgets.grid.Grid;
import org.eclipse.nebula.widgets.grid.GridColumn;
import org.eclipse.nebula.widgets.grid.GridItem;
import org.eclipse.swt.SWT;
import org.eclipse.swt.widgets.Composite;

import com.gx.obe.bind.view.ChangeListener;
import com.gx.obe.bind.view.RView;

public class FGrid<T> extends Grid implements RView<T> {
	
	private final List<Column<T>> columnList = new ArrayList<>();
	
	private final List<ChangeListener> changeListeners = new ArrayList<>();
	
	private T t;
	
	@SuppressWarnings("unchecked")
	public FGrid(Composite parent) {
		super(parent, SWT.H_SCROLL | SWT.V_SCROLL);
		this.setHeaderVisible(true);
		this.setItemHeight(30);
		this.addSelectionListener(org.eclipse.swt.events.SelectionListener.widgetSelectedAdapter(e -> {
			GridItem gridItem = (GridItem) e.item;
			t = (T) gridItem.getData();
			changeListeners.forEach(ChangeListener::change);
		}));
	}
	
	public GridColumn addIndexColumn(String name) {
		columnList.add(new IndexColumn<>());
		GridColumn gridColumn = new GridColumn(this, SWT.CENTER);
		gridColumn.setText(name);
		return gridColumn;
	}
	
	public GridColumn addColumn(String name, Function<T, String> function, int style) {
		columnList.add(new Column<>(function));
		GridColumn gridColumn = new GridColumn(this, style);
		gridColumn.setText(name);
		return gridColumn;
	}
	
	public void refresh(List<T> tList) {
		this.removeAll();
		if (tList != null) for (int i = 0; i < tList.size(); i++) {
			T t = tList.get(i);
			GridItem gridItem = new GridItem(this, SWT.NONE);
			gridItem.setData(t);
			for (int j = 0; j < columnList.size(); j++) {
				Column<T> column = columnList.get(j);
				if (column instanceof IndexColumn) {
					gridItem.setText(j, i + 1 + "");
				} else {
					gridItem.setText(j, Optional.ofNullable(column.function.apply(t)).orElse(""));
				}
			}
		}
	}
	
	private static class Column<T> {
		private Function<T, String> function;
		
		private Column(Function<T, String> function) {
			this.function = function;
		}
	}
	
	private static class IndexColumn<T> extends Column<T> {
		private IndexColumn() {
			super(null);
		}
	}
	
	@Override
	public void addChangeListener(ChangeListener changeListener) {
		this.changeListeners.add(changeListener);
	}
	
	@Override
	public T get() {
		return t;
	}
	
}
