package com.gx.obe.component.handler;

import java.util.HashMap;
import java.util.Map;
import java.util.function.Consumer;
import java.util.function.Function;

import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.widgets.Composite;

import com.gx.obe.bind.prop.Prop;
import com.gx.obe.bind.view.ChangeListener;
import com.gx.obe.bind.view.MView;
import com.gx.obe.component.utils.CompositeUtils;

public class RaidoHandler<T extends ISelection, R> implements MView<R> {
	
	private ISelection tab = ISelection.selection;
	
	private Composite composite;
	private Function<Composite, T> iSelectionFunction;
	
	private Prop<R> prop = new Prop<>();
	
	private Map<R, T> map = new HashMap<>();
	
	public RaidoHandler(Composite composite, Function<Composite, T> iSelectionFunction) {
		this.composite = composite;
		this.iSelectionFunction = iSelectionFunction;
	}
	
	public RaidoHandler<T, R> start() {
		CompositeUtils.disposeChildren(composite);
		return this;
	}
	
	public RaidoHandler<T, R> hand(R r, String text) {
		return hand(r, t -> t.setText(text));
	}
	
	public RaidoHandler<T, R> hand(R r, Consumer<T> setConsumer) {
		T t = iSelectionFunction.apply(composite);
		setConsumer.accept(t);
		t.addMouseListener(MouseListener.mouseUpAdapter($ -> prop.set(r)));
		map.put(r, t);
		return this;
	}
	
	public void end() {
		prop.bind(r -> {
			tab.setSelection(false);
			tab = map.get(r);
			tab.setSelection(true);
		});
		composite.layout();
		composite.getParent().layout();
	}
	
	@Override
	public void addChangeListener(ChangeListener changeListener) {
		prop.addChangeListener(changeListener);
	}
	
	@Override
	public R get() {
		return prop.get();
	}
	
	@Override
	public void set(R r) {
		prop.set(r);
	}
	
}
