package com.gx.obe.component.handler;

import java.util.List;
import java.util.function.BiConsumer;
import java.util.function.Consumer;
import java.util.function.Function;

import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.widgets.Composite;

import com.gx.obe.bind.call.Callback;
import com.gx.obe.component.utils.CompositeUtils;

public class TabHandler<T extends ISelection> {
	
	private ISelection tab;
	
	private final Composite composite;
	private final Function<Composite, T> iTabFunction;
	
	private Callback firstCall;
	
	public TabHandler(Composite composite, Function<Composite, T> iTabFunction) {
		this.composite = composite;
		this.iTabFunction = iTabFunction;
	}
	
	public TabHandler<T> start() {
		CompositeUtils.disposeChildren(composite);
		firstCall = null;
		return this;
	}
	
	public <R> TabHandler<T> hand(String text, Callback callback) {
		return hand(null, (t, r) -> t.setText(text), r -> callback.call());
	}
	
	public <R> TabHandler<T> hand(Consumer<T> setConsumer, Callback callback) {
		return hand(null, (t, r) -> setConsumer.accept(t), r -> callback.call());
	}
	
	public <R> TabHandler<T> hand(R r, BiConsumer<T, R> setBiConsumer, Consumer<R> consumer) {
		T t = iTabFunction.apply(composite);
		setBiConsumer.accept(t, r);
		t.addMouseListener(MouseListener.mouseUpAdapter($ -> {
			tab.setSelection(false);
			tab = t;
			tab.setSelection(true);
			consumer.accept(r);
		}));
		if (firstCall == null) firstCall = () -> consumer.accept(r);
		return this;
	}
	
	public void end() {
		composite.layout();
		composite.getParent().layout();
		CompositeUtils.getFirst(composite).ifPresent(t -> {
			tab = (ISelection) t;
			tab.setSelection(true);
			firstCall.call();
		});
	}
	
	public <R> TabHandler<T> listHand(List<R> tList, Function<R, String> getTestFunction, Consumer<R> consumer) {
		return listHand(tList, (t, r) -> t.setText(getTestFunction.apply(r)), consumer);
	}
	
	public <R> TabHandler<T> listHand(List<R> tList, BiConsumer<T, R> setBiConsumer, Consumer<R> consumer) {
		if (tList != null) tList.forEach(r -> hand(r, setBiConsumer, consumer));
		return this;
	}
	
}
