package com.gx.obe.component.menu;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;
import java.util.function.Consumer;
import java.util.function.Function;
import java.util.stream.Stream;

import org.eclipse.nebula.widgets.grid.Grid;
import org.eclipse.nebula.widgets.grid.GridColumn;
import org.eclipse.nebula.widgets.grid.GridItem;
import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Widget;

import com.gx.obe.bind.fun.Fun;
import com.gx.obe.bind.prop.Prop;
import com.gx.obe.util.utils.PredicateUtils;

public class ListMenu<T> extends Composite {
	
	protected Consumer<T> consumer = t -> {};
	
	private Prop<List<T>> tListProp = new Prop<>(new ArrayList<>());
	private Prop<Integer> menuWidth = new Prop<>(300);
	
	/**
	 * @author chenxw
	 * @param parent
	 * @param style
	 */
	public ListMenu(Composite parent, Function<T, String> menuNameFun, Function<Composite, Consumer<T>> function) {
		super(parent, SWT.NONE);
		setLayout(new GridLayout(2, false));
		
		Composite menuComposite = new Composite(this, SWT.NONE);
		menuComposite.setLayout(new GridLayout(1, false));
		menuComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, false, true, 1, 1));
		
		Grid grid = new Grid(menuComposite, SWT.V_SCROLL);
		grid.addSelectionListener(SelectionListener.widgetSelectedAdapter(this::widgetSelected));
		grid.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		tListProp.bind(l -> {
			grid.removeAll();
			l.stream().forEach(t -> {
				GridItem gridItem = new GridItem(grid, SWT.NONE);
				gridItem.setText(menuNameFun.apply(t));
				gridItem.setData(t);
				gridItem.setHeight(30);
			});
			Optional.of(grid).filter(Fun.of(Grid::getItemCount).toTest(PredicateUtils.higher(0))).ifPresent(g -> g.select(0));
			Optional.of(grid).map(Grid::getSelection).map(Stream::of).flatMap(Stream::findFirst).ifPresent(this::selectItemAction);
		});
		
		GridColumn gridColumn = new GridColumn(grid, SWT.NONE);
		menuWidth.bind(gridColumn::setWidth);
		
		Composite content = new Composite(this, SWT.NONE);
		content.setLayout(new FillLayout(SWT.HORIZONTAL));
		content.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		consumer = Optional.ofNullable(function).orElseGet(() -> p -> t -> {}).apply(content);
	}
	
	public void refresh(List<T> tList) {
		tListProp.set(tList);
	}
	
	public void setMenuWidth(int menuWidth) {
		this.menuWidth.set(menuWidth);
	}
	
	private void widgetSelected(SelectionEvent e) {
		selectItemAction(e.item);
	}
	
	private void selectItemAction(Widget item) {
		@SuppressWarnings("unchecked")
		T t = (T) item.getData();
		consumer.accept(t);
	}
	
	@Override
	protected void checkSubclass() {}
}
