package com.gx.obe.component.tabfolder;

import java.util.Objects;
import java.util.Optional;
import java.util.function.Consumer;
import java.util.function.Function;

import org.eclipse.swt.SWT;
import org.eclipse.swt.core.utils.WidgetUtils;
import org.eclipse.swt.core.widgets.ESepator;
import org.eclipse.swt.core.widgets.ImageButton;
import org.eclipse.swt.custom.StackLayout;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.MouseListener;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Composite;

import com.gx.obe.ColorConstants;
import com.gx.obe.bind.call.Callback;
import com.gx.obe.bind.prop.Prop;
import com.gx.obe.component.utils.CompositeUtils;
import com.swtdesigner.SWTResourceManager;

/**
 * @Description: 选项卡布局
 * @author chenxw
 */
public abstract class CTabFolder extends Composite {
	
	private final StackLayout stackLayout = new StackLayout();
	
	private final Composite itemComposite;
	private final Composite contentComposite;
	
	private CTabItem curTabItem;
	private Composite actionComposite;
	private int index;
	
	/**
	 * @author chenxw
	 * @param parent
	 * @param style
	 */
	public CTabFolder(Composite parent, int style) {
		super(parent, style);
		GridLayout gridLayout = new GridLayout(1, false);
		gridLayout.verticalSpacing = 0;
		gridLayout.marginHeight = 0;
		gridLayout.marginWidth = 0;
		setLayout(gridLayout);
		
		Prop<Boolean> showItemList = new Prop<Boolean>(false);
		
		itemComposite = new Composite(this, SWT.NONE);
		RowLayout rl_composite = new RowLayout(SWT.HORIZONTAL);
		rl_composite.marginBottom = 0;
		rl_composite.marginTop = 0;
		rl_composite.marginRight = 0;
		rl_composite.marginLeft = 0;
		itemComposite.setLayout(rl_composite);
		itemComposite.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		showItemList.bind(t -> WidgetUtils.setControlVisible(itemComposite, t));
		
		ESepator timeSep = new ESepator(this, SWT.HORIZONTAL);
		timeSep.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1));
		timeSep.setBackground(ColorConstants.SKIN_BG);
		timeSep.setLineWidth(3);
		showItemList.bind(t -> WidgetUtils.setControlVisible(timeSep, t));
		showItemList.bind(t -> layout());
		
		contentComposite = new Composite(this, SWT.NONE);
		contentComposite.setLayout(stackLayout);
		contentComposite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		
		actionComposite = new Composite(contentComposite, SWT.NONE);
		actionComposite.setLayout(new FillLayout(SWT.HORIZONTAL));
		
		init();
		
		showItemList.set(itemComposite.getChildren().length > 1);
	}
	
	public void notifyMouseUp() {
		CompositeUtils.notifyMouseUp(itemComposite);
	}
	
	public void refresh() {
		if (curTabItem != null) curTabItem.refresh();
		else CompositeUtils.notifyMouseUp(itemComposite);
	}
	
	/**
	 * @Description: 初始化
	 * @author chenxw
	 */
	protected abstract void init();
	
	/**
	 * @Description: 选项卡
	 * @author chenxw
	 */
	protected class CTabItem extends ImageButton {
		
		private Consumer<Composite> consumer = t -> {};
		
		private Callback callback = () -> {};
		
		private boolean action;
		private boolean refresh;
		
		private int index = 0;
		
		/**
		 * @author chenxw
		 * @param text
		 * @param function
		 */
		public CTabItem(String text) {
			super(itemComposite, SWT.NONE);
			
			this.setBorderColor(null);
			
			this.setForeground(SWTResourceManager.getColor(127, 127, 127));
			this.setBackgroundColor(SWTResourceManager.getColor(221, 221, 221));
			
			this.setActiveForeground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
			this.setActiveBgColor(SWTResourceManager.getColor(255, 153, 0));
			
			this.setSelectionForeground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
			this.setSelectionBgColor(SWTResourceManager.getColor(60, 138, 226));
			
			this.setLeftMargin(15);
			this.setRightMargin(15);
			this.setTopMargin(6);
			this.setBottomMargin(6);
			
			setText(text);
			setFont(SWTResourceManager.getFont("微软雅黑", 9, SWT.BOLD));
			addMouseListener(MouseListener.mouseUpAdapter(this::mouseUpAction));
		}
		
		public void action(Consumer<Composite> consumer) {
			this.consumer = Objects.requireNonNull(consumer);
			this.action = true;
		}
		
		public void refresh(Function<Composite, Callback> function) {
			this.callback = Objects.requireNonNull(function).apply(contentComposite);
			this.refresh = true;
			this.index = ++CTabFolder.this.index;
		}
		
		/**
		 * @Description: 鼠标点击事件
		 * @author chenxw
		 * @param e
		 */
		private void mouseUpAction(MouseEvent e) {
			Optional.ofNullable(curTabItem).ifPresent(t -> t.setSelection(false));
			this.setSelection(true);
			curTabItem = this;
			if (action) stackLayout.topControl = actionComposite;
			if (refresh) stackLayout.topControl = contentComposite.getChildren()[index];
			contentComposite.layout();
			refresh();
		}
		
		private void refresh() {
			if (action) {
				CompositeUtils.disposeChildren(actionComposite);
				consumer.accept(actionComposite);
				actionComposite.layout();
			}
			if (refresh) callback.call();
		}
		
	}
	
}
