package com.gx.obe.component.text;

import java.math.BigDecimal;
import java.util.Objects;
import java.util.Optional;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;

import com.gx.obe.bind.fun.Fun;
import com.gx.obe.bind.prop.Prop;
import com.gx.obe.bind.view.ChangeListener;
import com.gx.obe.bind.view.MView;
import com.gx.obe.components.core.util.MoneyUtils;
import com.swtdesigner.SWTResourceManager;

/**
 * @Description: 大写金额文本框
 * @author chenxw
 */
public class CapitalMoneyText extends Composite implements MView<BigDecimal> {
	
	private final Prop<BigDecimal> moneyProp = new Prop<>(BigDecimal.ZERO);
	
	private final Prop<Color> moneyColorProp = new Prop<>(SWTResourceManager.getColor(90, 147, 52));
	
	private final Prop<Boolean> editableProp = new Prop<>(true);
	
	public CapitalMoneyText(Composite parent, String unit) {
		super(parent, SWT.NONE);
		setBackground(SWTResourceManager.getColor(SWT.COLOR_WHITE));
		setBackgroundMode(SWT.INHERIT_FORCE);
		
		Objects.requireNonNull(unit);
		
		GridLayout gridLayout = new GridLayout(1, false);
		gridLayout.verticalSpacing = 0;
		gridLayout.marginHeight = 0;
		gridLayout.marginWidth = 0;
		setLayout(gridLayout);
		
		Composite composite = new Composite(this, SWT.NONE);
		composite.setBackground(SWTResourceManager.getColor(238, 238, 238));
		GridLayout gl_composite = new GridLayout(3, false);
		gl_composite.marginHeight = 6;
		gl_composite.marginWidth = 20;
		gl_composite.horizontalSpacing = 20;
		composite.setLayout(gl_composite);
		composite.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		
		Label lblNewLabel = new Label(composite, SWT.NONE);
		lblNewLabel.setForeground(SWTResourceManager.getColor(188, 188, 188));
		lblNewLabel.setFont(SWTResourceManager.getFont("微软雅黑", 11, SWT.NORMAL));
		lblNewLabel.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, false, 1, 1));
		lblNewLabel.setText("小写");
		
		MoneyText text = new MoneyText(composite, SWT.RIGHT, unit);
		text.setFont(SWTResourceManager.getFont("微软雅黑", 18, SWT.NORMAL));
		text.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		moneyProp.bindBidirectional(text);
		moneyColorProp.bind(text::setForeground);
		editableProp.bind(text::setEditable);
		
		Label lblNewLabel_2 = new Label(composite, SWT.NONE);
		lblNewLabel_2.setForeground(SWTResourceManager.getColor(160, 160, 160));
		lblNewLabel_2.setFont(SWTResourceManager.getFont("微软雅黑", 10, SWT.NORMAL));
		lblNewLabel_2.setText(unit);
		
		Composite composite_1 = new Composite(this, SWT.NONE);
		composite_1.setBackground(SWTResourceManager.getColor(245, 245, 245));
		GridLayout gl_composite_1 = new GridLayout(2, false);
		gl_composite_1.marginHeight = 6;
		gl_composite_1.marginWidth = 20;
		gl_composite_1.horizontalSpacing = 20;
		composite_1.setLayout(gl_composite_1);
		GridData gd_composite_1 = new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1);
		gd_composite_1.heightHint = 41;
		composite_1.setLayoutData(gd_composite_1);
		
		Label lblNewLabel_1 = new Label(composite_1, SWT.NONE);
		lblNewLabel_1.setForeground(SWTResourceManager.getColor(188, 188, 188));
		lblNewLabel_1.setFont(SWTResourceManager.getFont("微软雅黑", 11, SWT.NORMAL));
		lblNewLabel_1.setLayoutData(new GridData(SWT.RIGHT, SWT.CENTER, false, true, 1, 1));
		lblNewLabel_1.setText("大写");
		
		Text text_1 = new Text(composite_1, SWT.RIGHT);
		text_1.setFont(SWTResourceManager.getFont("微软雅黑", 14, SWT.NORMAL));
		text_1.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, true, 1, 1));
		text_1.setEditable(false);
		moneyProp.bind(Fun.of(this::moneyToCapital).toWrite(text_1::setText));
		moneyColorProp.bind(text_1::setForeground);
		
	}
	
	/**
	 * @Description: 金额转大写
	 * @author chenxw
	 * @param money
	 * @return
	 */
	private String moneyToCapital(BigDecimal money) {
		return Optional.ofNullable(money).map(BigDecimal::doubleValue).map(MoneyUtils::amountToChinese).orElse("");
	}
	
	@Override
	public void addChangeListener(ChangeListener changeListener) {
		moneyProp.addChangeListener(changeListener);
	}
	
	@Override
	public void set(BigDecimal t) {
		moneyProp.set(t);
	}
	
	/**
	 * @Description: 设置金额字体颜色
	 * @author chenxw
	 * @param moneyColor
	 */
	public void setMoneyColor(Color moneyColor) {
		moneyColorProp.set(moneyColor);
	}
	
	/**
	 * @Description: 设置是否可编辑
	 * @author chenxw
	 * @param editable
	 */
	public void setEditable(Boolean editable) {
		editableProp.set(editable);
	}
	
	@Override
	public BigDecimal get() {
		return moneyProp.get();
	}
	
	@Override
	protected void checkSubclass() {}
	
}