package com.gx.obe.component.verify;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.VerifyEvent;
import org.eclipse.swt.events.VerifyListener;
import org.eclipse.swt.widgets.Text;

/**
 * @Description: 金额验证接听
 * @author chenxw
 */
public class MoneyVerifyListener implements VerifyListener {
	
	private boolean negativeSupport;
	private boolean tenThousand;
	
	/**
	 * @author chenxw
	 * @param negativeSupport
	 * @param tenThousand
	 */
	public MoneyVerifyListener(boolean negativeSupport, boolean tenThousand) {
		this.negativeSupport = negativeSupport;
		this.tenThousand = tenThousand;
	}
	
	@Override
	public void verifyText(VerifyEvent event) {
		if (event.keyCode == SWT.BS || event.keyCode == SWT.DEL) {
			event.doit = true;
			return;
		}
		event.doit = false;
		char myChar = event.character;
		Text text = (Text) event.widget;
		String textStr = text.getText();
		
		// 当通过代码set值时
		if (myChar == 0) {
			event.doit = true;
		}
		// 当输入负号时
		else if (myChar == '-') {
			// 支持负数
			if (negativeSupport) {
				// 当前文本中没有负号
				if (textStr.indexOf("-") == -1) {
					// 第一次输入
					if (event.start == 0) {
						event.doit = true;
					}
				}
			}
		}
		// 当输入小数点时
		else if (myChar == '.') {
			// 当前文本中没有小数点
			if (textStr.indexOf(".") == -1) {
				// 不是第一次输入
				if (event.start != 0) {
					event.doit = true;
				}
			}
		}
		// 当输入数字时
		else if ((myChar >= '0' && myChar <= '9')) {
			// 第一次输入
			if (textStr.isEmpty()) {
				event.doit = true;
			} else {
				String integralPart = textStr;
				int indexOf = textStr.indexOf('.');
				// 当前文本中有小数点时截取整数部分
				if (indexOf != -1) {
					integralPart = textStr.substring(0, indexOf);
				}
				// 当前整数部分是以负号开始时去掉负号
				if (integralPart.startsWith("-")) {
					integralPart = integralPart.substring(1);
				}
				// 去掉负号后的整数部分长度不能超过金额限定长度
				if (integralPart.length() < (tenThousand ? 10 : 14)) {
					event.doit = true;
				}
			}
		}
	}
	
}
